package com.guagua.base.util;

import java.io.IOException;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Type;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.guagua.base.BusinessException;

/**
 * 配置文件工具 beta
 * 
 * @author 徐峰 2011-12-12 version 0.01
 */
public class ConfigUtil {
	static Logger logger = LoggerFactory.getLogger(ConfigUtil.class);
	
	final static ScheduledExecutorService scheduledExecutorService = Executors.newScheduledThreadPool(5);

	/**
	 * 读取配置文件，映射成一个对象
	 * @param <T> 最终要映射成的对象
	 * @param fileName 配置文件名
	 * @param typeOfT 存放映射类型 
	 * @return 读取的对象
	 */
	public static <T> T read(String fileName, Type typeOfT) {
		try {
			final String str = readText(fileName);
			return getObjectByText(typeOfT, str);
		} catch (Exception e) {
			throw new RuntimeException("配置文件读取错误 " + fileName, e);
		}
	}

	/**
	 * 得到输入文本代表的对象 
	 * @param typeOfT 对象类型
	 * @param str 输入文本
	 * @return 转换后的对象
	 */
	private static <T> T getObjectByText(Type typeOfT, final String str) {
		logger.debug("#read_config_content 配置文件内容：{}", str);
		return new Gson().fromJson(removeComments(str), typeOfT);
	}

	/**
	 * 读取配置文件 内容
	 * @param fileName 配置文件名
	 * @return 配置文件内容
	 * @throws IOException
	 */
	public static String readText(String fileName) throws IOException {
		URL url = EnvironmentUtil.findConfig(fileName, null);
		logger.info("#read_config_name 读取配置文件 {} path: {}", fileName, url);
		String text = EnvironmentUtil.getUrlContent(url);
		return text;
	}
	
	/**
	 * 配置参数绑定 静态对象 
	 * @param clazz 要绑定的静态对象类名
	 * @param columnName 要绑定的字段名
	 * @param typeOfT 绑定的字段类型
	 * @param fileName 绑定的配置文件名
	 */
	public static void bind(Class<?> clazz, String columnName, Type typeOfT, String fileName) {
		bind(new BindTarget(null, clazz, columnName, null), typeOfT, fileName);
	}
	
	/**
	 * 配置参数绑定 指定对象上的字段
	 * @param obj 指定对象
	 * @param columnName 对象上的字段名
	 * @param typeOfT 绑定的字段类型
	 * @param fileName 绑定的配置文件名
	 */
	public static void bind(Object obj, String columnName, Type typeOfT, String fileName) {
		bind(new BindTarget(obj, obj.getClass(), columnName, null), typeOfT, fileName);
	}
	
	/**
	 * 将配置文件绑定到一个静态方法上，每次配置文件更新时自动调用该方法。 将配置文件转换后的对象作为参数传给该方法（要绑定的方法必须只有一个类型为：typeOfT 的参数）
	 * @param clazz 要绑定到那个类的静态方法
	 * @param methodName 绑定的方法名 
	 * @param typeOfT 绑定的方法上的参数类型
	 * @param fileName 配置文件名
	 */
	public static void bindMethod(Class<?> clazz, String methodName, Type typeOfT, String fileName) {
		bind(new BindTarget(null, clazz, null, methodName), typeOfT, fileName);
	}
	
	/**
	 * 将配置文件绑定到指定对象的方法上，每次配置文件更新时自动调用该方法。 将配置文件转换后的对象作为参数传给该方法（要绑定的方法必须只有一个类型为：typeOfT 的参数）
	 * @param obj 指定对象
	 * @param methodName 对象上的方法名
	 * @param typeOfT 绑定的方法上的参数类型
	 * @param fileName 绑定的配置文件名
	 */
	public static void bindMethod(Object obj, String methodName, Type typeOfT, String fileName) {
		bind(new BindTarget(obj, obj.getClass(), null, methodName), typeOfT, fileName);
	}
	
	/** 绑定的配置文件缓存 用来检查文件是否变化 */
	private static Map<String, String> BIND_CONFIG_CACHE = new HashMap<String, String>();  
	
	/** 绑定检查的周期（单位分钟） */
	final static int period = 10;
	final static TimeUnit timeUnit = TimeUnit.MINUTES;
//	final static TimeUnit timeUnit = TimeUnit.SECONDS;

	/**
	 * 配置参数绑定 基础实现方法
	 * @param obj 绑定的对象
	 * @param field 要绑定的字段
	 * @param typeOfT 绑定的数据类型
	 * @param fileName 配置文件名
	 */
	private static void bind(final BindTarget bindTarget, final Type typeOfT, final String fileName) {
		try {
			verify(fileName, bindTarget);
			invokeBind(typeOfT, fileName, bindTarget);
			scheduledExecutorService.scheduleAtFixedRate(new Runnable() {
				@Override
				public void run() {
					invokeBind(typeOfT, fileName, bindTarget);
				}
			}, period, period, timeUnit);
		}
		catch (Exception e) {
			logger.error("配置文件绑定失败 " + fileName, e);
		}
	}
	
	/**
	 * 绑定前的安全检查,防止重复绑定
	 * @param fileName
	 * @param field
	 * @param obj
	 */
	private static void verify(String fileName, BindTarget bindTarget) {
		if (BIND_CONFIG_CACHE.containsKey(bindTarget.key)) {
			throw new RuntimeException("#config_bind_error 该字段已经绑定过了！ " + bindTarget.key + " " + BIND_CONFIG_CACHE.get(bindTarget.key));
		}
	}

	/**
	 * 执行配置文件绑定对象操作
	 * @param typeOfT 对象类型
	 * @param fileName 配置文件名
	 * @param field 要绑定的字段
	 * @param obj 字段所在的对象
	 * @throws Exception
	 */
	private static void invokeBind(Type typeOfT, String fileName, BindTarget bindTarget){
		try {
			String oldText = BIND_CONFIG_CACHE.get(bindTarget.key);
			String text = readText(fileName);
			if (text.equals(oldText)) {
				logger.info("#config_bind 配置文件内容相等，不需要设置");
			}
			else {
				logger.info("#config_bind 配置文件内容更新，重新读取");
				BIND_CONFIG_CACHE.put(bindTarget.key, text);
				Object result = getObjectByText(typeOfT, text);
				bindTarget.bind(result);
			}
		} catch (Exception e) {
			logger.error("#config_bind_error 配置文件绑定 定时读取错误！ {}", fileName + " " + bindTarget.key, e);
		}
	}

	/** 换行符 */
	static final String LINE_BREAK = "\n";
	
	/**
	 * 移除配置文件中的注释
	 * @param content
	 * @return
	 */
	private static String removeComments(String content) {
		StringBuilder result = new StringBuilder();
		for (String line : content.split(LINE_BREAK)) {
			if (!line.trim().startsWith("//")) {
				result.append(line);
				result.append(LINE_BREAK);
			}
		}
		String out = result.toString();
		return out;
	}
	
	/**
	 * 绑定目标
	 * @author 徐峰
	 */
	static class BindTarget {
		final Field field;
		final Method method;
		final Object obj;
		final String key;

		/**
		 * @param obj 要绑定到哪个实体对象， 为null绑定到静态对象上
		 * @param clazz 要绑定的类
		 * @param fieldName 要绑定的字段名（字段名和方法名只能有一个）
		 * @param methodName 要绑定的方法名（字段名和方法名只能有一个）
		 * @throws Exception 
		 */
		BindTarget(Object obj, Class<?> clazz, String fieldName, String methodName) {
			Field field = null;
			Method method = null;
			
			try {
				bindOk:
				if (fieldName != null && methodName == null) {
					field = clazz.getDeclaredField(fieldName);
					field.setAccessible(true);
				}
				else if (methodName != null && fieldName == null) {
					for(Method eachMethod :clazz.getDeclaredMethods()) {
						if (eachMethod.getName().equals(methodName) && eachMethod.getParameterTypes().length == 1) {
							method = eachMethod;
							method.setAccessible(true);
							break bindOk;
						}
					}
					logger.error("绑定目标初始化失败 ，方法名未找到:" + clazz + " " + methodName);
				}
				else {
					logger.error("绑定目标初始化失败 ， 字段 或 方法名 能且仅能绑定一个 fieldName:" + fieldName + " methodName:" + methodName);
				}
			} catch (Throwable e) {
				logger.error("绑定目标初始化失败 " + obj + " " + clazz + " " + fieldName + " " + methodName, e);
			}
			
			this.obj = obj;
			this.field = field;
			this.method = method;
			this.key = clazz.getName() + "@" + 
					(obj != null ? Integer.toHexString(obj.hashCode()) : "null") + 
					"-" + field + "-" + method; 
		}
		
		/**
		 * 向指定目标上绑定一个对象
		 * @param bindValue 要绑定的值
		 * @throws Exception
		 */
		void bind(Object bindValue) throws Exception {
			if (field != null) {
				field.set(obj, bindValue);
			}
			else if (method != null) {
				method.invoke(obj, bindValue);
			}
			else {
				throw new BusinessException("未设置绑定对象" + this);
			}
		}
		
		@Override
		public String toString() {
			return obj + " " + field + " " + method + " " + key;
		}
	}
}

 
