package com.guagua.base.util;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.dom4j.Document;
import org.dom4j.io.SAXReader;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.guagua.base.BusinessException;

/** 
 * 运行环境相关处理
 * 
 * @author 徐峰 2011-9-1 version 0.01
 */
public class EnvironmentUtil {
	static Logger logger = LoggerFactory.getLogger(EnvironmentUtil.class);
	private static final Pattern VERSION_PATTERN = Pattern.compile("([0-9][0-9\\.\\-]*).*\\.jar");

	/**
	 * 返回jar包相关描述信息
	 */
	public static String getJarInfo(Class<?> clazz) {
		final Set<String> pathSet = getResourcesPath(clazz.getName().replace('.', '/') + ".class");
		return clazz.getSimpleName() + " " + pathSet.size() + " " + getVersion(clazz, "0.0.0") + " " + pathSet;
	}

	/**
	 * 重复检查
	 * 
	 * @param path
	 */
	public static void checkDuplicate(String path) {
		Set<String> files = getResourcesPath(path);
		if (files.size() > 1) {
			logger.error("#Duplicate class " + path + " in " + files.size() + " jar " + files);
		}
	}

	/**
	 * jar包 重复引入检查
	 * 
	 * @param class
	 */
	public static void checkDuplicate(Class<?> clazz) {
		checkDuplicate(clazz.getName().replace('.', '/') + ".class");
	}

	/**
	 * 得到指定资源的所有路径 集合
	 */
	public static Set<String> getResourcesPath(String path) {
		Set<String> files = new HashSet<String>();
		Enumeration<URL> urls;
		try {
			urls = Thread.currentThread().getContextClassLoader().getResources(path);
			while (urls.hasMoreElements()) {
				URL url = urls.nextElement();
				if (url != null) {
					String file = url.getFile();
					if (file != null && file.length() > 0) {
						files.add(file);
					}
				}
			}
		} catch (Throwable e) {
			logger.error(e.getMessage(), e);
		}
		return files;
	}

	/**
	 * 获取指定类相关jar包的版本号
	 * @param clazz
	 * @param defaultVersion
	 * @return
	 */
	public static String getVersion(Class<?> clazz, String defaultVersion) {
		try {
			String version;
			
			// 首先查找MANIFEST.MF规范中的版本号
			version = clazz.getPackage().getImplementationVersion();
			if (version != null && !version.isEmpty()) {
				return version;
			}
			
			version = clazz.getPackage().getSpecificationVersion();
			if (version != null && !version.isEmpty()) {
				return version;
			}
			
			// 如果MANIFEST.MF规范中没有版本号，基于jar包名获取版本号
			String file = clazz.getProtectionDomain().getCodeSource().getLocation().getFile();
//			logger.info("#token_jarFileName " + clazz + " " + file);
			if (file != null && file.length() > 0) {
				Matcher matcher = VERSION_PATTERN.matcher(file);
				while (matcher.find() && matcher.groupCount() > 0) {
					version = matcher.group(1);
				}
			}
			
			// 返回版本号，如果为空返回缺省版本号
			return version == null || version.length() == 0 ? defaultVersion : version;
		} catch (Throwable e) { // 防御性容错
			// 忽略异常，返回缺省版本号
			logger.error(e.getMessage(), e);
			return defaultVersion;
		}
	}

	
	/**
	 * 按顺序在特定的配置文件中查找指定名称的文件，返回最先找到的路径，都没找到返回null，具体顺序为：
	 * 1.首先，在资源目录（classes）下 badu_init.xml文件中指定的路径下查找
	 * 2.其次，在 应用目录下的 “WEB-INF/baduHome/” 路径中寻找
	 * 3.如果配置文件名以 remote开头 在websystem.17guagua.com/info?names={name} 寻找
	 * 4.在 应用目录下的 “WEB-INF/baduHome/remote/” 路径中寻找
	 * 5.其次，在 资源目录下查找
	 * 6.最后，在 clazz.getResource(name) 中寻找
	 * @param name 要寻找的文件名
	 */
	public static URL findConfig(String name, Class<?> clazz) {
		URL url = null;
		
		//1.首先，在资源目录（classes）下 badu_init.xml文件中指定的路径下查找
		if (CONFIG_PATH != null) {
			File file = new File(CONFIG_PATH + File.separator + name);
			if (file.isFile()) {
				try {
					url = new URL("File:" + file.getCanonicalPath());
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			}
		}
		
		//2.其次，在 应用目录下的 “WEB-INF/baduHome/” 路径中寻找
		if (url == null) {
			final String pathname = getApplicationPath() + File.separator + "baduHome/" + name;
//			logger.debug("寻找目录：" + pathname);
			File file = new File(pathname);
			if (file.isFile()) {
				try {
					url = new URL("File:" + file.getCanonicalPath());
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			}
		}
		
		//3. 如果配置文件名以 remote开头。
		//在websystem.17guagua.com/info?names={name} 寻找,如果找到写入WEB-INF/baduHome/remote/ 路径中
		if (url == null && name.startsWith("remote_")) {
			HttpRequestInfo httpRequestInfo = null;
			try {
				httpRequestInfo = getHttpRequestInfo(name).setTimeout(5000);
				HttpResponseInfo request = HttpUtil.request(httpRequestInfo);
				String text = request.getSuccessContent();
				if (!request.getHeaderFields().containsKey("badu_server")) {
					throw new BusinessException("#remote_error fail 失败！链接被劫持 " + text);
				}
				logger.info("#config_access_url_success url配置文件访问成功 " + httpRequestInfo.getUrl());
				try {
					//url数据获取成功，将url文件写入本地文件系统，以备链接故障时使用
					String dir = getRemoteBackDir();
					new File(dir).mkdirs();
					File file = new File(dir + name);
					FileWriter writer = new FileWriter(file);
					writer.write(text);
					writer.close();
				} catch (Exception e) {
					logger.debug("write fail WEB-INF/baduHome/remote/" + name + " " + e);
				}
			} catch (Exception e) {
				logger.debug("http config fail {} {}", name + " " + e, httpRequestInfo);
			}
		}
		
		//在 资源目录下的 “WEB-INF/baduHome/remote/” 路径中寻找 (url 配置文件的备份 文件，防止在url服务停止时无法启动的问题）
		if (url == null) {
			File file = new File(getRemoteBackDir() + name);
			if (file.isFile()) {
				try {
					url = new URL("File:" + file.getCanonicalPath());
				} catch (Exception e) {
					logger.error(e.getMessage(), e);
				}
			}
		}
		
		//其次，在 资源目录下查找
		if (url == null) {
			url = Thread.currentThread().getContextClassLoader().getResource(name);
		}
		
		//最后，在 clazz.getResource(name) 中寻找
		if (url == null && clazz != null) {
			url = clazz.getResource(name);
		}
		
		logger.info("读取配置文件：" + name + " 位置 " + url);
		return url;
	}

	/**
	 * 得到远端配置文件备份目录
	 * @return
	 */
	private static String getRemoteBackDir() {
		return getApplicationPath() + File.separator + "baduHome/remote/";
	}

	private static String consumerKey;
	private static String consumerSecret;
	private static String tokenSecret = "";
	private static String remoteUrl;
	
	/**
	 * 生成访问需要的 请求对象
	 * @param name 要访问的参数名
	 * @return http访问对象
	 * @throws Exception
	 */
	private static HttpRequestInfo getHttpRequestInfo(String name) throws Exception {
		readBaseConfig();
		HttpRequestInfo request = new HttpRequestInfo().setUrl(remoteUrl).addParameter("name", name);
		request.addParameter(
				SecurityUtil.getOauth1Parameter(request.getUrl(), request.getMethod(), request.getParameterList(), consumerKey, consumerSecret, tokenSecret));
		return request;
	}
	
	/**
	 * 读取基础配置文件
	 * @throws Exception 
	 */
	private static void readBaseConfig(){
		if (consumerKey != null) {
			return;
		}
		
		consumerKey = "badu";
		consumerSecret = "baud(all_commone-quanxian_xf_userkye%$&)2011-12-15publish";
//		remoteUrl = "http://websystem.17guagua.com/info";
		remoteUrl = "http://control.passport.17guagua.com/info/config";
		try {
			URL url = findConfig("base_util_config.conf", EnvironmentUtil.class);
			InputStream in = url.openStream();
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String line;
			while ((line = br.readLine()) != null) {
				logger.debug(line);
				if (!line.startsWith("//") && !line.startsWith("#")) {
					String[] split = line.trim().split("=");
					logger.debug("{}", split);
					String name = split[0].trim();
					if (name.equals("consumerKey")) {
						consumerKey = split[1].trim();
					}
					if (name.equals("consumerSecret")) {
						consumerSecret = split[1].trim();
					}
					if (name.equals("remoteUrl")) {
						remoteUrl = split[1].trim();
					}
				}
			}
		} catch (Exception e) {
			logger.info("读取配置文件出错。");
		}
		logger.info("基础参数：source:{}, key:{}", consumerKey, consumerSecret);
		logger.info("基础参数：remoteUrl:{}", remoteUrl);
	}


	/** 资源目录下  badu_init.xml 文件所定义配置文件目录 */
	public final static String CONFIG_PATH = getConfigPath();
	private static String getConfigPath() {
		try {
			InputStream in = EnvironmentUtil.class.getResourceAsStream("/badu_init.xml");
			Document doc = (new SAXReader()).read(in);
			return doc.getRootElement().getText();
		} catch (Throwable e) {
			logger.info("#token /badu_init.xml 配置文件打开失败，忽略读取该文件 " + e);
			return null;
		}
	}
	
	/**
	 * 得到当前应用路径。（当前类文件所在第一个/WEB-INF/ 文件夹）
	 * @return 成功返回 当前类所在文件夹的上一层目录，失败返回null
	 */
	public static String getApplicationPath() {
		String file = EnvironmentUtil.class.getProtectionDomain().getCodeSource().getLocation().getFile();
//		logger.info(file);
		final String fileFlag = "file:";
		if (file.startsWith(fileFlag)) {
			file = file.substring(fileFlag.length());
		}
		final String applicationFlag = "WEB-INF";
		return file.substring(0, file.lastIndexOf(applicationFlag) + applicationFlag.length());
	}

	/**
	 * 得到url中的内容
	 * @param url 要获取的url
	 * @return 该url的内容 （文本）
	 * @throws IOException
	 */
	public static String getUrlContent(URL url) throws IOException {
		InputStream in = url.openStream();
		try {
			StringBuilder sb = new StringBuilder();
			BufferedReader br = new BufferedReader(new InputStreamReader(in));
			String line;
			while ((line = br.readLine()) != null) {
				sb.append(line);
				sb.append("\n");
			}
			final String str = sb.toString();
			return str;
		} finally {
			in.close();
		}
	}

	public static String getConsumerKey() {
		return consumerKey;
	}

	public static String getConsumerSecret() {
		return consumerSecret;
	}
	
	
}
