package com.guagua.base.util;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.Writer;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.guagua.base.function.Function1;
import com.guagua.base.function.Function2;


/**
 * 文件操作相关工具
 * @author xufeng
 */
public class FileUtil {
	static Logger log = LoggerFactory.getLogger(FileUtil.class);

	/**
	 * 获取一个文件实例
	 * @param pathname
	 * @return
	 */
	public static File getInstanceFile(String pathname) {
		File file = new File(pathname);
		if (log.isDebugEnabled()) {
			log.debug(fileLog(file));
		}
		return file;
	}

	/**
	 * 获取一个文件实例
	 * @param uri 
	 * @return
	 */
	private static File getInstanceFile(URI uri) {
		File file = new File(uri);
		if (log.isDebugEnabled()) {
			log.debug(fileLog(file));
		}
		return file;
	}
	
	/**
	 * 获取一个写文件的 writer 的实例
	 * @param pathname 要写入的文件名
	 * @return 实例
	 * @throws IOException
	 */
	public static Writer getInstanceWriter(String pathname) throws IOException {
		return new FileWriter(getInstanceFile(pathname));
	}

	/**
	 * 获得文件存在相关信息
	 * @param file
	 */
	private static String fileLog(File file) {
		if (file.isDirectory()) {
			return "已打开文件夹：" + file.getAbsoluteFile();
		}
		String state = file.exists() ? "已打开文件:" : "请求文件不存在："; 
		return state + file.getAbsolutePath();
	}
	
	public static File readFile(String name, Class<?> clazz) throws Exception {
		URL resource = clazz.getResource(name);
		if(resource == null) {
			throw new Exception("文件目录：" + ClassLoader.getSystemResource(".") + "的类" + clazz + "文件不存在：" + name);
		}
		return  getInstanceFile(resource.toURI());
	}

	public static void main(String[] args) {
		Class<?> clazz = FileUtil.class;
		URL re = ClassLoader.getSystemResource("./");
		log.debug(re.toString());
		InputStream in = clazz.getResourceAsStream("xxxx");
		log.debug(in.toString());
	}
	/**
	 * 遍历文本文件中的所有行
	 * @param pathname 文件路径名
	 * @param fun1 行处理方法
	 * @throws Exception
	 */
	public static void eachLines(String pathname, Function1<Void, String> fun1) throws Exception {
		FileReader reader = new FileReader(getInstanceFile(pathname)); 
		BufferedReader br = new BufferedReader(reader);
		String line;
		while ((line = br.readLine()) != null) {
			fun1.apply(line);
		}
		br.close();
	}

	/**
	 * 获得指定文件的所有内容
	 * @param pathname 指定文件名
	 * @return 文件所有内容
	 * @throws Exception
	 */
	public static String getText(String pathname) throws Exception {
		StringBuilder sb = new StringBuilder();
		FileReader reader = new FileReader(getInstanceFile(pathname)); 
		BufferedReader br = new BufferedReader(reader);
		String line;
		while ((line = br.readLine()) != null) {
			sb.append(line);
			sb.append("\n");
		}
		br.close();
		return sb.toString();
	}

	/**
	 * 遍历处理指定目录下的所有目录中的文件。 将每个目录下的文件传给 fn处理
	 * @param fileRoot 根目录 如果fileRoot为空则直接返回。
	 * @param fn 处理函数 <String 绝对目录名, List<File> 目录下的文件列表>
	 * @throws Exception
	 */
	public static void eachDir(File fileRoot, Function2<Void, String, List<File>> fn ) throws Exception {
		if (fileRoot == null) {
			return;
		}
		List<File> fileList = new ArrayList<File>();
		List<File> dirList = new ArrayList<File>();
		for (File file : fileRoot.listFiles()) {
			if (file.isFile()) {
				fileList.add(file);
			}
			else if (file.isDirectory()) {
				dirList.add(file);
			}
		}
		fn.apply(fileRoot.getAbsolutePath() + File.separator, fileList);
		for (File file : dirList) {
			eachDir(file, fn);
		}
	}
	
	/**
	 * 遍历处理指定目录下的所有目录中的文件。
	 * @param fileRoot 根目录
	 * @param fn 处理函数 <File 每个文件>
	 * @throws Exception
	 */
	public static void eachFile(File fileRoot, Function1<Void, File> fn) throws Exception {
		if (fileRoot.isFile()) {
			fn.apply(fileRoot);
		}
		else if (fileRoot.isDirectory()) {
			for (File file : fileRoot.listFiles()) {
				eachFile(file, fn);
			}
		}
		else {
			log.info("non-existent： " + fileRoot);
		}
	}

	public static void copy(File src, File dest) throws Exception {
		new File(dest.getParent()).mkdirs();
		FileInputStream fin = new FileInputStream(src);
		FileOutputStream fout = new FileOutputStream(dest);
		
		byte[] bs = new byte[1024];
		int len = 0;
		while (len != -1) {
			fout.write(bs, 0, len);
			len = fin.read(bs);
		}
		fout.close();
		fin.close();
	}
	
	
}
