package com.guagua.base.util;

import static java.net.URLEncoder.encode;

import java.net.InetSocketAddress;
import java.net.Proxy;
import java.net.SocketAddress;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * http 请求信息
 * 只负责存储信息，不参与具体逻辑
 * @author 徐峰
 *
 */
public class HttpRequestInfo {
	static Logger logger = LoggerFactory.getLogger(HttpRequestInfo.class);
	
	/** 连接的url */
	String url;
	
	/** http 查询参数串 */
	List<NameValuePair> parameterList = new ArrayList<NameValuePair>();
	
	/** http 请求属性 */
	List<NameValuePair> propertyList = new ArrayList<NameValuePair>();
	
	/** 连接超时时间（毫秒） */
	int timeout = 10000;
	
	/** 代理设置 */
	Proxy proxy = null;
	
	/** 请求方法 */
	RequestMethod method = RequestMethod.GET;
	
	/** 应答的编码集 */
	String responseCharsetName = "utf-8";
	
	public String getResponseCharsetName() {
		return responseCharsetName;
	}

	public HttpRequestInfo setResponseCharsetName(String responseCharsetName) {
		this.responseCharsetName = responseCharsetName;
		return this;
	}

	public RequestMethod getMethod() {
		return method;
	}

	/** 设置请求方法 */
	public HttpRequestInfo setMethod(RequestMethod method) {
		this.method = method;
		return this;
	}

	/**
	 * 创建一个http连接实例
	 * @param url
	 * @return
	 */
	public static HttpRequestInfo newInstance(String url) {
		return new HttpRequestInfo().setUrl(url);
	}
	
	public HttpRequestInfo setUrl(String url) {
		Assert.isTrue(url.startsWith("http://") || url.startsWith("https://"), "");
		this.url = url;
		return this;
	}
	
	/**
	 * 添加一个 请求参数
	 * @param name
	 * @param value
	 * @return
	 * @throws Exception 
	 */
	public HttpRequestInfo addParameter(String name, String... values) throws Exception {
		for (String value : values) {
//			parameterList.add(new NameValuePair(encode(name, "utf-8"), encode(value, "utf-8")));
			parameterList.add(new NameValuePair(name, value));
		}
		return this;
	}
	
	/**
	 * 添加多个 请求参数
	 * @param pairs
	 * @return
	 */
	public HttpRequestInfo addParameter(NameValuePair... pairs) {
		for (NameValuePair pair : pairs) {
			parameterList.add(pair);
		}
		return this;
	}
	
	/**
	 * 添加一个 请求属性
	 * @param name
	 * @param value
	 * @return
	 */
	public HttpRequestInfo addProperty(String name, String... values) {
		for (String value : values) {
			propertyList.add(new NameValuePair(name, value));
		}
		return this;
	}
	
	/**
	 * 设置超时时间
	 * @param timeout
	 * @return
	 */
	public HttpRequestInfo setTimeout(int timeout) {
		this.timeout = timeout;
		return this;
	}

	/**
	 * 是否存在代理
	 * @return
	 */
	public boolean isProxy() {
		return proxy != null;
	}
	
	/**
	 * 设置代理地址
	 * @param socketAddress
	 * @return
	 */
	public HttpRequestInfo proxySocketAddress(SocketAddress socketAddress) {
		proxy = new Proxy(Proxy.Type.HTTP, socketAddress);
		return this;
	}

	/**
	 * 设置代理地址 如果ip 或端口的错误，则忽略该配置
	 * @param ip
	 * @param port
	 * @return
	 */
	public HttpRequestInfo proxySocketAddress(String ip, int port) {
		if (StringUtil.isEmpty(ip) || port < 0) {
			return this;
		}
		proxySocketAddress(new InetSocketAddress(ip, port));
		return this;
	}

	
	/**
	 * 得到代理对象
	 * @return 代理对象
	 */
	public Proxy getProxy() {
		return proxy;
	}

	/**
	 * http请求属性列表
	 * @return
	 */
	public List<NameValuePair> getPropertyList() {
		return propertyList;
	}

	public String getUrl() {
		return url;
	}

	public List<NameValuePair> getParameterList() {
		return parameterList;
	}
	
	/**
	 * 得到查询参数串
	 * @return
	 * @throws Exception 
	 */
	public String getParameterString() throws Exception {
		StringBuilder parameterString = new StringBuilder();
        for (NameValuePair pair : getParameterList()) {
        	parameterString.append(encode(pair.name, "utf-8")).append("=").append(encode(StringUtil.toStringNoNull(pair.value), "utf-8")).append("&");
        }
        if (parameterString.length() == 0) {
        	return "";
        }
        else {
        	return parameterString.substring(0, parameterString.length() - 1);
        }
	}

	public int getTimeout() {
		return timeout;
	}
	
	@Override
	public String toString() {
		return proxy + " " + url + (method == RequestMethod.POST ? ", parameterList:" + parameterList : "") + ", propertyList" + propertyList;
	}
}
