package com.guagua.base.util;

import static java.net.URLEncoder.encode;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicInteger;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.codec.binary.Base64;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.guagua.base.BusinessException;
import com.guagua.base.ThreadContent;

/**
 * 安全相关工具
 */
public class SecurityUtil {
	static Logger logger = LoggerFactory.getLogger(SecurityUtil.class);
	
	private final static AtomicInteger seq = new AtomicInteger();
	
	/** 签名字段参数名 */
	public final static String OAUTH_SIGNATURE = "oauth_signature";
	
	/** 访问的客户端 标识 参数名 */
	public final static String oauthConsumerKey = "oauth_consumer_key";

	
	/**
	 * 得到客户端安全信息
	 * @param request 请求信息
	 * @param tokenSecret 用户身份密钥（暂时为空 ""）
	 * @return 
	 * @throws Exception
	 */
	public static NameValuePair[] getOauth1Parameter(HttpRequestInfo request, String tokenSecret) throws Exception {
		return getOauth1Parameter(request.url, request.getMethod(), request.getParameterList(), EnvironmentUtil.getConsumerKey(), EnvironmentUtil.getConsumerSecret(), tokenSecret);
	}
	
	/**
	 * 得到客户端安全信息
	 * @param url 要请求的url
	 * @param method
	 * @param pairs 请求参数
	 * @param consumerKey 客户端唯一标识
	 * @param consumerSecret 密钥
	 * @param tokenSecret 用户身份密钥（暂时为空 ""）
	 * @return oauth 接口需要添加的参数
	 * @throws Exception
	 */
	public static NameValuePair[] getOauth1Parameter(String url, RequestMethod method, List<NameValuePair> pairs, String consumerKey, String consumerSecret, String tokenSecret) throws Exception {
		Map<String, String> parameterMap = new HashMap<String, String>();
		for (NameValuePair pair : pairs) {
			parameterMap.put(pair.name, pair.value);
		}
		
		if (parameterMap.containsKey(OAUTH_SIGNATURE)){
			//如果签名字段已存在，则跳过
			return new NameValuePair[0];
		}
		
		ArrayList<NameValuePair> currentPairs = new ArrayList<NameValuePair>(pairs);
		
		int index = url.indexOf("?");
		if (index > 0) {
			String queryStr = url.substring(index + 1);
			for (String strPair: queryStr.split("&")) {
				String[] ar = strPair.split("=");
				currentPairs.add(new NameValuePair(ar[0], ar[1]));
			}
			url = url.substring(0, index);
		}
		
		String timestamp = System.currentTimeMillis() + "";

		List<NameValuePair> addPairs = new ArrayList<NameValuePair>(4);
		if (!parameterMap.containsKey(oauthConsumerKey)){
			addPairs.add(new NameValuePair(oauthConsumerKey, consumerKey));
		}
		String oauthTimestamp = "oauth_timestamp";
		if (!parameterMap.containsKey(oauthTimestamp)){
			addPairs.add(new NameValuePair(oauthTimestamp, timestamp));
		}
		String oauthNonce = "oauth_nonce";
		if (!parameterMap.containsKey(oauthNonce)){
			addPairs.add(new NameValuePair("oauth_nonce", timestamp + "_" + seq.getAndIncrement()));
		}
		currentPairs.addAll(addPairs);
		
		String signatureBaseString = getSignatureBaseString(url, method.name(), currentPairs);
	    String signatureValue = getSignatureValue(consumerSecret, tokenSecret, signatureBaseString);
	    
		addPairs.add(new NameValuePair(OAUTH_SIGNATURE, signatureValue));
		
		NameValuePair[] result = new NameValuePair[addPairs.size()];
		for(int i = 0; i < result.length; i++){
			result[i] = addPairs.get(i);
		}
		return result;
	}

	/**
	 * 得到签名值
	 * @param consumerSecret 
	 * @param tokenSecret
	 * @param signatureBaseString
	 * @return
	 * @throws NoSuchAlgorithmException
	 * @throws InvalidKeyException
	 */
	private static String getSignatureValue(String consumerSecret, String tokenSecret, String signatureBaseString)
			throws NoSuchAlgorithmException, InvalidKeyException {
		Mac mac = Mac.getInstance("HmacSHA1");
	    String mykey = consumerSecret + "&" + tokenSecret;
		SecretKeySpec secretKey = new SecretKeySpec(mykey.getBytes(), "HmacSHA1");
		mac.init(secretKey);
		byte[] digest = mac.doFinal(signatureBaseString.getBytes());
		byte[] bs = Base64.encodeBase64(digest);
		String signatureValue = new String(bs);
		logger.debug("signature 基础串 {} 签名 {} key " + mykey, signatureBaseString, signatureBaseString);
		return signatureValue;
	}

	/**
	 * 得到Oauth1.0验证所需的基础串
	 * @param url 
	 * @param method
	 * @param currentPairs
	 * @return
	 * @throws Exception
	 */
	private static String getSignatureBaseString(String url, String method, List<NameValuePair> currentPairs) throws Exception {
		Collections.sort(currentPairs, new Comparator<NameValuePair>() {
			@Override
			public int compare(NameValuePair o1, NameValuePair o2) {
				return o1.name.compareTo(o2.name);
			}
		});
		
		logger.trace("#pairs_sort {}", currentPairs);
		
		StringBuilder sb = new StringBuilder(method).append("&").append(encode(url, "utf-8")).append("&");
		for (NameValuePair pair : currentPairs) {
			sb.append(encode(pair.name, "utf-8")).append("%3D").append(encode(pair.value, "utf-8")).append("%26");
		}
		String signatureBaseString = sb.substring(0, sb.length() - 3);
//		logger.debug("#signatureBaseString 签名计算基础串 {}", signatureBaseString );
		return signatureBaseString;
	}
	
	
	/**
	 * 验证 Oauth 的签名 
	 * 简化方法： verifyOauthSignature(consumerKeySecretMap)（需要ThreadContent 类支持） 
	 * @param url 访问的url
	 * @param method 访问方法
	 * @param parameter 参数
	 * @param consumerSecret 服务器端密钥
	 * @param tokenSecret  客户密钥
	 * @param ip 来源ip（仅做输出日志用）
	 * @throws Exception
	 */
	public static void verifyOauth1Signature(String url, String method, List<NameValuePair> parameter, String consumerSecret, String tokenSecret, String ip) throws Exception {
		List<NameValuePair> executeParameter = new ArrayList<NameValuePair>(parameter.size());
		String signatureVlaue = "";
		for (NameValuePair pair : parameter) {
			if (pair.name.equals(OAUTH_SIGNATURE)) {
				signatureVlaue = pair.value;
			}
			else {
				executeParameter.add(pair);
			}
		}
		
		String signatureBaseString = getSignatureBaseString(url, method, executeParameter);
		String computationalSignatureValue = getSignatureValue(consumerSecret, tokenSecret, signatureBaseString);
		
		if (!computationalSignatureValue.equals(signatureVlaue)) {
			logger.info("#oauth_signature_error 签名错误" +
					" secret:" + consumerSecret + "&" + tokenSecret + 
					" signatureBaseString:" + signatureBaseString + 
					" signature: " + signatureVlaue + " != " + computationalSignatureValue + 
					" " + ip);
			throw new BusinessException(400, "Invalid signature 签名错误");
		}
	}
	
	/**
	 * 验证 Oauth 的签名 需要 ThreadContent 类支持
	 * @throws Exception 
	 */
	@SuppressWarnings("unchecked")
	public static void verifyOauthSignature(Map<String, String> consumerKeySecretMap) throws Exception {
		HttpServletRequest request = ThreadContent.request();
		List<NameValuePair> parameter = new ArrayList<NameValuePair>();
		Enumeration<String> names = request.getParameterNames();
		while (names.hasMoreElements()) {
			String name = names.nextElement();
			parameter.add(new NameValuePair(name, request.getParameter(name)));
		}
		
		String key = request.getParameter(oauthConsumerKey);
		String realKey = SetUtil.findMatch(key, consumerKeySecretMap.keySet());
		String consumerSecret = consumerKeySecretMap.get(realKey);
		String tokenSecret = "";
		verifyOauth1Signature(request.getScheme() + "://" + request.getServerName() + request.getRequestURI(), request.getMethod(), parameter, consumerSecret, tokenSecret, ThreadContent.clientIp());
	}
}
